//----------------------------------------------------------------------------

//    

//-----------------------  : ------------------------------

//     RC-5 (  - SAA3010).
//   :  INT0   
//    2.

//    - ST_IDLE.  
//    2,    
//.   ,   ST_IDLE  ST_DONE,
//     ,  
//,    Power Down.   
//  ,   .

//    -,   
//     INT0.     INT0,
//   Sleep,    2  
//    T_SAMPLE / 2 ( ST_SMP2).

//         
//(       1).   
//      .
//          
//  .   ,   ,  
//    (   ST_IDLE,  
//,  INT0   ,    Power Down).
//         T_SAMPLE
//      ( ST_SMP1).

//        
//.        
//  .       T_SAMPLE,
//    ( ),   
//  (   ST_IDLE   ).
//        .
//   ,       
// System  Command,    ST_DONE.  
//     ,    
//  ST_IDLE.

//        GetCom().
//  ,     ST_DONE (  ),
//   .     SysUsed, 
//   .    ,
//   .    REM_NO.
// GetCom()    ST_DONE  ST_IDLE.

//----------------------------------------------------------------------------

#include "main.hpp"
#include "remote.hpp"
#include "keyboard.hpp"

//---------------------    RC-5: ------------------------

#define RC5_LENGTH 14 //  
#define RC5_SLOT 1778 //  RC-5, 

struct RC5Struct //  RC-5
{
  uint16_t RC5Command : 6;
  uint16_t RC5System  : 5;
  uint16_t RC5Control : 1;
  uint16_t RC5Start   : 2;
};

#define PRE      64 // 
#define T_SAMPLE ((uint8_t)(RC5_SLOT / 2 * F_CLK / PRE / 1000000)) //Ts
#define DEL2     (256 - T_SAMPLE)     //  Ts
#define DEL1     (256 - T_SAMPLE / 2) //  1/2 Ts

enum { ST_IDLE, ST_SMP1, ST_SMP2, ST_TO, ST_DONE }; // 

//----------------------------------------------------------------------------
//----------------------------  TRemote: --------------------------------
//----------------------------------------------------------------------------

//------------------------  : ---------------------------

uint8_t volatile  TRemote::State;
uint16_t volatile TRemote::System;
uint8_t volatile  TRemote::Command;

//-------------  inline- ( ): ------------

inline void TRemote::TmrIntEnable(uint8_t delay)
{
  TCNT2 = delay;
  TIFR2 = (1 << TOV2);
  TIMSK2 |= (1 << TOIE2);
}

inline void TRemote::TmrIntDisable(void)
{
  TIMSK2 &= ~(1 << TOIE2);
}

inline void TRemote::ExtInt0Enable(void)
{
  EICRA = 0;                       // INT0   
  EIFR = (1 << INTF0);
  EIMSK |= (1 << INT0);
}

inline void TRemote::ExtIntEnable(void)
{
  EICRA = (1 << ISC00);            // INT0   
  EIFR = (1 << INTF0);
  EIMSK |= (1 << INT0);
}

inline void TRemote::ExtIntDisable(void)
{
  EIMSK &= ~(1 << INT0);
}

//----------------------------- : ---------------------------------

TRemote::TRemote(uint16_t sys)
{
  Pin_LED.DirOut(PIN_LOW);         //  
  Pin_IR.DirIn(PIN_PULL);          //  
  SysUsed = sys;                   //  
  Command = REM_NO;                //  
  TCCR2B = (1 << CS22);            // CK/64   2
  State = ST_IDLE;                 //    
  TmrIntEnable(DEL2);              //  
}

//-------------     : --------------

#pragma vector = INT0_vect
__interrupt void InputIR(void)
{
  TRemote::ExtIntDisable();        //  
  TRemote::TmrIntEnable(DEL1);     //  
  TRemote::State = ST_SMP2;        //    
  TRemote::Pin_LED = 1;            // 
  SMCR = 0;                        //sleep disable
}

//------------------    2: ------------------------

#pragma vector = TIMER2_OVF_vect
__interrupt void TimerIR(void)
{
  static uint8_t BitCnt = RC5_LENGTH; //  
  static bool PreVal = 1;             //   
  static union
  {
    uint16_t RC5Code;              // 
    RC5Struct RC5Bits;             //   
  };

  bool Val = TRemote::Pin_IR;      //  

  //    :
  if(TRemote::State == ST_TO)
  {
    TRemote::State = ST_IDLE;      // 
  }

  //     :
  if(TRemote::State == ST_SMP1)
  {
    PreVal = Val;                  //    
    TRemote::ExtIntEnable();       //  
    TRemote::TmrIntEnable(DEL2);   //  
    TRemote::State = ST_TO;        //    
    //   :
    if(!--BitCnt)
    {
      TRemote::System = RC5Bits.RC5System;   //  
      TRemote::Command = RC5Bits.RC5Command; //  
      if(RC5Bits.RC5Control)        //RC5Control = 1,
        TRemote::Command |= RC5_CB; //   RC5_CB
      TRemote::State = ST_DONE;     // 
    }
  }

  //     :
  if(TRemote::State == ST_SMP2)
  {
    //  :
    if(Val != PreVal)
    {
      RC5Code <<= 1;               //  
      if(!Val) RC5Code |= 1;       //   = 0,  = 1
      TRemote::TmrIntEnable(DEL2); //  
      TRemote::State = ST_SMP1;    //    
    }
    else
    {
      TRemote::State = ST_IDLE;    //  
    }
  }

  //  ( ):
  if((TRemote::State == ST_DONE) ||
    (TRemote::State == ST_IDLE))
  {
    BitCnt = RC5_LENGTH;           //  
    PreVal = 1;                    //     
    TRemote::TmrIntDisable();      //  
    TRemote::ExtInt0Enable();      //    0
    TRemote::Pin_LED = 0;          // 
    SMCR = (1 << SM1) | (1 << SE); //Power Down enable
  }
}

//------------------------   : ------------------------------

uint8_t TRemote::GetCom(void)
{
  uint8_t c = REM_NO;
  if((State == ST_DONE) &&
     (System == SysUsed))
  {
    State = ST_IDLE;
    c = Command;
  }
  return(c);
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
